/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2020 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : Config_UART0_user.c
* Version      : 1.0.0
* Device(s)    : R7F100GLGxFA
* Description  : This file implements device driver for Config_UART0.
* Creation Date: 2021-11-30
***********************************************************************************************************************/
/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code for pragma. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_userdefine.h"
#include "Config_UART0.h"
/* Start user code for include. Do not edit comment generated here */
#include "r_cg_userdefine.h"
#include <string.h>
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
extern volatile uint8_t * gp_uart0_tx_address;    /* uart0 transmit buffer address */
extern volatile uint16_t g_uart0_tx_count;        /* uart0 transmit data number */
extern volatile uint8_t * gp_uart0_rx_address;    /* uart0 receive buffer address */
extern volatile uint16_t g_uart0_rx_count;        /* uart0 receive data number */
extern volatile uint16_t g_uart0_rx_length;       /* uart0 receive data length */
/* Start user code for global. Do not edit comment generated here */
volatile bool g_UART0_sendend = false;
static uint8_t  g_recv_data[261];
static volatile bool g_soft_recv_overrun = false;

static uint16_t getUARTRecvSize(void);
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: R_Config_UART0_Create_UserInit
* Description  : This function adds user code after initializing UART0.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_Config_UART0_Create_UserInit(void)
{
    /* Start user code for user init. Do not edit comment generated here */
    /* End user code. Do not edit comment generated here */
}

/***********************************************************************************************************************
* Function Name: r_Config_UART0_callback_sendend
* Description  : This function is a callback function when UART0 finishes transmission.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void r_Config_UART0_callback_sendend(void)
{
    /* Start user code for r_Config_UART0_callback_sendend. Do not edit comment generated here */
  g_UART0_sendend = 0x01;
    /* End user code. Do not edit comment generated here */
}

/***********************************************************************************************************************
* Function Name: r_Config_UART0_callback_receiveend
* Description  : This function is a callback function when UART0 finishes reception.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void r_Config_UART0_callback_receiveend(void)
{
    /* Start user code for r_Config_UART0_callback_receiveend. Do not edit comment generated here */
    /* End user code. Do not edit comment generated here */
}

/***********************************************************************************************************************
* Function Name: r_Config_UART0_callback_error
* Description  : This function is a callback function when UART0 reception error occurs.
* Arguments    : err_type -
*                    error type info
* Return Value : None
***********************************************************************************************************************/
static void r_Config_UART0_callback_error(uint8_t err_type)
{
    /* Start user code for r_Config_UART0_callback_error. Do not edit comment generated here */
    /* End user code. Do not edit comment generated here */
}

/***********************************************************************************************************************
* Function Name: r_Config_UART0_callback_softwareoverrun
* Description  : This function is a callback when UART0 receives an overflow data.
* Arguments    : rx_data -
*                    receive data
* Return Value : None
***********************************************************************************************************************/
static void r_Config_UART0_callback_softwareoverrun(uint16_t rx_data)
{
    /* Start user code for r_Config_UART0_callback_softwareoverrun. Do not edit comment generated here */
  g_soft_recv_overrun = 1;
    /* End user code. Do not edit comment generated here */
}

/***********************************************************************************************************************
* Function Name: r_Config_UART0_interrupt_send
* Description  : This function is UART0 send interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
#pragma vector = INTST0_vect
__interrupt static void r_Config_UART0_interrupt_send(void)
{
    if (g_uart0_tx_count > 0U)
    {
        TXD0 = *gp_uart0_tx_address;
        gp_uart0_tx_address++;
        g_uart0_tx_count--;
    }
    else
    {
        r_Config_UART0_callback_sendend();
    }
}

/***********************************************************************************************************************
* Function Name: r_Config_UART0_interrupt_receive
* Description  : This function is UART0 receive interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
#pragma vector = INTSR0_vect
__interrupt static void r_Config_UART0_interrupt_receive(void)
{
    uint8_t rx_data;

    rx_data = RXD0;

    if (g_uart0_rx_length > g_uart0_rx_count)
    {
        *gp_uart0_rx_address = rx_data;
        gp_uart0_rx_address++;
        g_uart0_rx_count++;

        if (g_uart0_rx_length == g_uart0_rx_count)
        {
            r_Config_UART0_callback_receiveend();
        }
    }
    else
    {
        r_Config_UART0_callback_softwareoverrun(rx_data);
    }
}

/***********************************************************************************************************************
* Function Name: r_Config_UART0_interrupt_error
* Description  : This function is UART0 error interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
#pragma vector = INTSRE0_vect
__interrupt static void r_Config_UART0_interrupt_error(void)
{
    uint8_t err_type;

    *gp_uart0_rx_address = RXD0;
    err_type = (uint8_t)(SSR01 & 0x0007U);
    SIR01 = (uint16_t)err_type;
    r_Config_UART0_callback_error(err_type);
}

/* Start user code for adding. Do not edit comment generated here */

/**********************************************************************************************************************
 * Function name : getUARTRecvSize
 *********************************************************************************************************************/
static uint16_t getUARTRecvSize(void)
{
    uint16_t size = 0;

    DI();

    size = g_uart0_rx_count;

    EI();

    return size;
}

/**********************************************************************************************************************
 * Function name : r_ClearUARTRecvBuff
 *********************************************************************************************************************/
void r_ClearUARTRecvBuff(void)
{
    g_uart0_rx_count = 0;

    g_soft_recv_overrun = false;

    R_Config_UART0_Receive(g_recv_data, sizeof(g_recv_data));
}

/**********************************************************************************************************************
 * Function name : r_AsyncRecvPacketData
 *********************************************************************************************************************/
/*
#define START_SECTION_USER_APLICATION
#include "memmap.h"*/
R_RFD_FAR_FUNC e_ret_t r_AsyncRecvPacketData(uint8_t *p_cmd_type, uint8_t rdata[])
{
    uint16_t recv_size = 0;
    uint16_t len = 0;
    uint16_t i;
    uint8_t cmd_type = 0;
    uint8_t calc_sum = 0;

    recv_size = getUARTRecvSize();
    if (recv_size < 5)
    {
        return ENUM_RET_STS_RECEIVING; // pPbgM
    }

    if (g_soft_recv_overrun)
    {
        return ENUM_RET_ERR_PACKET;
    }

    if (g_recv_data[0] != 0x01)
    {
        return ENUM_RET_ERR_PACKET;
    }

    len = g_recv_data[1] << 8 | g_recv_data[2];
    if (recv_size < (3 + len)) // 3 = STARTR[h + f[^(2byte)  len = R}h + f[^ + `FbNT
    {
        return ENUM_RET_STS_RECEIVING; // pPbgM
    }

    // Check Length
    cmd_type = g_recv_data[3];
    if (((cmd_type == COMMAND_WRITE_BOOT1 || cmd_type == COMMAND_WRITE_TEMP) 
        && (len != 0x0102)) || ((cmd_type == COMMAND_START || cmd_type == COMMAND_END) && (len != 0x02)))
    {
        return ENUM_RET_ERR_PACKET;
    }

    // Check Sum
    for (i = 0; i < (len - 1); i++)
    {
        calc_sum += g_recv_data[i + 3];
    }

    if ((((cmd_type == COMMAND_WRITE_BOOT1) || (cmd_type == COMMAND_WRITE_TEMP)) 
        && (calc_sum != g_recv_data[260])) || (((cmd_type == COMMAND_START) || (cmd_type == COMMAND_END)) && (calc_sum != g_recv_data[4])))
    {
        return ENUM_RET_ERR_PACKET;
    }

    *p_cmd_type = cmd_type; // Set command code
    if ((cmd_type == COMMAND_WRITE_BOOT1) || (cmd_type == COMMAND_WRITE_TEMP))
    {
        memcpy(rdata, &g_recv_data[4], 256);
    }

    return ENUM_RET_STS_OK;
}
/*
#define END_SECTION_USER_APLICATION
#include "memmap.h"*/
/* End user code. Do not edit comment generated here */

