/***********************************************************************/
/*                                                                     */
/*  FILE        :Main.c                                                */
/*  DATE        :                                                      */
/*  DESCRIPTION :Main Program                                          */
/*  CPU TYPE    :                                                      */
/*                                                                     */
/*  NOTE:THIS IS A TYPICAL EXAMPLE.                                    */
/*                                                                     */
/***********************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_rfd_common_api.h"
#include "r_rfd_common_control_api.h"
#include "r_rfd_common_get_api.h"
#include "r_rfd_code_flash_api.h"
#include "r_rfd_extra_area_api.h"
#include "r_smc_entry.h"
#include "r_cg_userdefine.h"
#include <string.h>

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
void main(void);
static void userApplicationLoop(void);
static e_ret_t updateLoop(void);
static void errorLedOn(void);
/***********************************************************************************************************************
* Function Name: main
* Description  : This function implements main function.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/

#pragma section const COPY_FLAG
const uint32_t __far g_copy_end = COPY_FLAG_COMPLETE;
#pragma section

void main(void)
{
    e_ret_t ret_value = ENUM_RET_STS_OK;

    R_Config_UART0_Start();
    EI();

    if (g_copy_end != COPY_FLAG_COMPLETE)
    {
        ret_value = r_rfd_initialize();

        if (ENUM_RET_STS_OK == ret_value)
        {
            P4_bit.no1 = LED_ON; 
            ret_value = r_cmd_end(); // Copy TEMPORARY area to EXECUTE area
            if (ENUM_RET_STS_OK == ret_value)
            {
                P4_bit.no1 = LED_OFF; 
                ret_value = r_RequestBootSwap();
            }
        }
    }

    if (ENUM_RET_STS_OK == ret_value)
    {
        r_ClearUARTRecvBuff();

        while (1)
        {
	    userApplicationLoop();

            ret_value = updateLoop();
            if (ENUM_RET_STS_OK != ret_value)
            {
                break;
            }
        }
    }
    errorLedOn();
    while (1)
        ;
}

/**********************************************************************************************************************
 * Function name : userApplicationLoop
 *********************************************************************************************************************/
#define START_SECTION_USER_APPLICATION
#include "memmap.h"
static void userApplicationLoop(void)
{
        static uint32_t count = 0;
            if (++count == 100000)
            {
                // Blink LED 1
                P0_bit.no3 = ~P0_bit.no3;
                count = 0;
            }
}
#define END_SECTION_USER_APPLICATION

/**********************************************************************************************************************
 * Function name : updateLoop
 *********************************************************************************************************************/
static e_ret_t updateLoop(void)
{
    static bool f_control_mode = UNPROGRAMMABLE_MODE;
    static bool is_initialized = false;
    static uint32_t write_start_address = BT1_START_ADDRESS;
    static uint32_t write_start_temp_address = TEMPORARY_START_ADDRESS;
    uint32_t write_end_address = BT1_END_ADDRESS;
    uint32_t write_end_temp_address = TEMPORARY_END_ADDRESS;
    static uint8_t rdata[261] = {0};
    uint8_t cmd_type = 0;
    e_ret_t ret_value = ENUM_RET_STS_OK;

    if (is_initialized == false)
    {
        ret_value = r_rfd_initialize();
        if (ret_value != ENUM_RET_STS_OK)
        {
            return ret_value;
        }
        is_initialized = true;
    }

    ret_value = r_AsyncRecvPacketData(&cmd_type, rdata);
    if (ret_value == ENUM_RET_ERR_PACKET)
    {
        return ret_value;
    }
    else if (ret_value == ENUM_RET_STS_OK)
    {
        r_ClearUARTRecvBuff();

        switch (cmd_type)
        {
        case COMMAND_START:
            if (UNPROGRAMMABLE_MODE == f_control_mode)
            {
                P0_bit.no2 = LED_ON; // START command received
                ret_value = r_cmd_start();
                f_control_mode = PROGRAMMABLE_MODE;
                P0_bit.no2 = LED_OFF; 
            }
            break;

        case COMMAND_WRITE_BOOT1:
            if (UNPROGRAMMABLE_MODE != f_control_mode)
            {
                P4_bit.no3 = LED_ON;

                ret_value = r_CF_MemoryWrite(&write_start_address, write_end_address, rdata);
                if (ret_value == ENUM_RET_STS_OK)
                {
                    r_SendACK();
                    P4_bit.no3 = LED_OFF;
                }
            }
            break;

        case COMMAND_WRITE_TEMP:
            if (UNPROGRAMMABLE_MODE != f_control_mode)
            {
                P4_bit.no2 = LED_ON; 
		
                ret_value = r_CF_MemoryWrite(&write_start_temp_address, write_end_temp_address, rdata);
                if (ret_value == ENUM_RET_STS_OK)
                {
                    r_SendACK();
                    P4_bit.no2 = LED_OFF; 
                }
            }
            break;

        case COMMAND_END:
            if (UNPROGRAMMABLE_MODE != f_control_mode)
            {
                P7_bit.no7 = LED_ON; 
		P4_bit.no1 = LED_ON; 
                ret_value = r_cmd_end(); // Copy TEMPORARY area to EXECUTE area
                P4_bit.no1 = LED_OFF; 
                r_SendACK();
                ret_value = r_RequestBootSwap();
            }
            break;

        default:
            // received invalid packet
            ret_value = ENUM_RET_ERR_PACKET;
            break;
        }

        if (ret_value != ENUM_RET_STS_OK)
        {
            return ret_value;
        }
    }

    return ENUM_RET_STS_OK;
}

/**********************************************************************************************************************
 * Function name : errorLedOn
 *********************************************************************************************************************/
static void errorLedOn(void)
{
    P0_bit.no3 = LED_OFF;
    P0_bit.no2 = LED_OFF;
    P4_bit.no3 = LED_OFF;
    P4_bit.no2 = LED_OFF;
    P7_bit.no7 = LED_OFF;
    P4_bit.no1 = LED_OFF;
    P3_bit.no1 = LED_ON; // error status
    P7_bit.no6 = LED_OFF;
}

/**********************************************************************************************************************
 * Function name : r_rfd_initialize
 *********************************************************************************************************************/
#define START_SECTION_SMP_CF
#include "memmap.h"
R_RFD_FAR_FUNC e_ret_t r_rfd_initialize(void)
{
    e_rfd_ret_t rfd_status_flag = R_RFD_ENUM_RET_STS_OK;
    e_ret_t ret_value = ENUM_RET_STS_OK;

    if (VALUE_U01_MASK0_1BIT == HIOSTOP)
    {
        rfd_status_flag = R_RFD_Init(CPU_FREQUENCY);
        if (R_RFD_ENUM_RET_STS_OK != rfd_status_flag)
        {
            ret_value = ENUM_RET_ERR_PARAMETER;
        }
    }
    else
    {
        ret_value = ENUM_RET_ERR_CONFIGURATION;
    }
    return ret_value;
}
#define END_SECTION_SMP_CF
#include "memmap.h"

/**********************************************************************************************************************
 * Function name : r_cmd_start
 *********************************************************************************************************************/
#define START_SECTION_SMP_CF
#include "memmap.h"
R_RFD_FAR_FUNC e_ret_t r_cmd_start(void)
{
    e_ret_t ret_value = ENUM_RET_STS_OK;

    DI();
    ret_value = r_CF_RangeErase(BT1_START_ADDRESS, BT1_END_ADDRESS); // Erase boot cluster 1
    EI();
    if (ENUM_RET_STS_OK == ret_value)
    {
        DI();
        ret_value = r_CF_RangeErase(TEMPORARY_START_ADDRESS, TEMPORARY_END_ADDRESS); // Erase temporary area
        EI();

        if (ENUM_RET_STS_OK == ret_value)
        {
            r_SendACK();
        }
    }

    return ret_value;
}
#define END_SECTION_SMP_CF
#include "memmap.h"

/**********************************************************************************************************************
 * Function name : r_cmd_end
 *********************************************************************************************************************/
#define START_SECTION_SMP_CF
#include "memmap.h"
R_RFD_FAR_FUNC e_ret_t r_cmd_end(void)
{
    e_ret_t ret_value = ENUM_RET_STS_OK;

    DI();
    ret_value = r_CF_RangeErase(EXECUTE_START_ADDRESS, EXECUTE_END_ADDRESS); // Erase execute area
    EI();
    if (ENUM_RET_STS_OK != ret_value)
    {
        return ret_value;
    }

    ret_value = r_CF_TempCopy();

    return ret_value;
}
#define END_SECTION_SMP_CF
#include "memmap.h"

/**********************************************************************************************************************
 * Function name : r_CF_RangeErase
 *********************************************************************************************************************/
#define START_SECTION_SMP_CF
#include "memmap.h"
R_RFD_FAR_FUNC e_ret_t r_CF_RangeErase(uint32_t start_addr, uint32_t end_addr)
{
    e_ret_t ret_value = ENUM_RET_STS_OK;
    uint32_t erase_addr = 0;

    for (erase_addr = start_addr; erase_addr <= end_addr; erase_addr += CF_BLOCK_SIZE)
    {
        ret_value = r_CF_EraseBlock(erase_addr);

        if (ENUM_RET_STS_OK != ret_value)
        {
            break;
        }
    }

    return (ret_value);
}
#define END_SECTION_SMP_CF
#include "memmap.h"

/**********************************************************************************************************************
 * Function name : r_CF_EraseBlock
 *********************************************************************************************************************/
#define START_SECTION_SMP_CF
#include "memmap.h"
R_RFD_FAR_FUNC e_ret_t r_CF_EraseBlock(uint32_t start_addr)
{
    e_rfd_ret_t rfd_ret_status = R_RFD_ENUM_RET_STS_OK;
    e_ret_t ret_status = ENUM_RET_STS_OK;
    e_ret_t ret_value = ENUM_RET_STS_OK;
    uint16_t block_number = (uint16_t)(start_addr >> VALUE_U08_SHIFT_ADDR_TO_BLOCK_CF);

    rfd_ret_status = R_RFD_SetFlashMemoryMode(R_RFD_ENUM_FLASH_MODE_CODE_PROGRAMMING);
    if (R_RFD_ENUM_RET_STS_OK != rfd_ret_status)
    {
        ret_value = ENUM_RET_ERR_MODE_MISMATCHED;
    }

    if (ENUM_RET_STS_OK == ret_value)
    {
        R_RFD_EraseCodeFlashReq(block_number);
        ret_status = r_CheckCFDFSequencerEnd();
        if (ENUM_RET_STS_OK != ret_status)
        {
            ret_value = ENUM_RET_ERR_ERASE;
        }
    }

    rfd_ret_status = R_RFD_SetFlashMemoryMode(R_RFD_ENUM_FLASH_MODE_UNPROGRAMMABLE);
    if (R_RFD_ENUM_RET_STS_OK != rfd_ret_status)
    {
        ret_value = ENUM_RET_ERR_MODE_MISMATCHED;
    }

    return ret_value;
}
#define END_SECTION_SMP_CF
#include "memmap.h"

/**********************************************************************************************************************
 * Function name : r_CF_WriteVerifySequence
 *********************************************************************************************************************/
#define START_SECTION_SMP_CF
#include "memmap.h"
R_RFD_FAR_FUNC e_ret_t r_CF_WriteVerifySequence(uint32_t write_start_addr, uint16_t write_data_length, uint8_t __near *write_data)
{
    e_ret_t ret_value = ENUM_RET_STS_OK;

    ret_value = r_CF_WriteData(write_start_addr, write_data_length, write_data);
    if (ENUM_RET_STS_OK == ret_value)
    {
        ret_value = r_CF_VerifyData(write_start_addr, write_data_length, write_data);
    }

    return ret_value;
}
#define END_SECTION_SMP_CF
#include "memmap.h"

/**********************************************************************************************************************
 * Function name : r_CF_WriteData
 *********************************************************************************************************************/
#define START_SECTION_SMP_CF
#include "memmap.h"
R_RFD_FAR_FUNC e_ret_t r_CF_WriteData(uint32_t start_addr, uint16_t write_data_length, uint8_t __near *write_data)
{
    uint16_t count;

    if (R_RFD_ENUM_RET_STS_OK != R_RFD_SetFlashMemoryMode(R_RFD_ENUM_FLASH_MODE_CODE_PROGRAMMING))
    {
        return ENUM_RET_ERR_MODE_MISMATCHED;
    }

    /******************************************************************************************************************
     * WRITE
     *****************************************************************************************************************/
    for (count = 0u; count < write_data_length; count += 4u)
    {
        R_RFD_WriteCodeFlashReq(start_addr + count, &write_data[count]);

        if (ENUM_RET_STS_OK != r_CheckCFDFSequencerEnd())
        {
            return ENUM_RET_ERR_WRITE;
        }
    }

    /******************************************************************************************************************
     * Set unprogrammable mode
     *****************************************************************************************************************/
    if (R_RFD_ENUM_RET_STS_OK != R_RFD_SetFlashMemoryMode(R_RFD_ENUM_FLASH_MODE_UNPROGRAMMABLE))
    {
        return ENUM_RET_ERR_MODE_MISMATCHED;
    }

    return ENUM_RET_STS_OK;
}
#define END_SECTION_SMP_CF
#include "memmap.h"

/**********************************************************************************************************************
 * Function name : r_CF_VerifyData
 *********************************************************************************************************************/
#define START_SECTION_SMP_CF
#include "memmap.h"
R_RFD_FAR_FUNC e_ret_t r_CF_VerifyData(uint32_t start_addr, uint16_t data_length, uint8_t __near *write_data)
{
    e_ret_t ret_value = ENUM_RET_STS_OK;
    uint16_t i = 0;
    uint32_t flash_addr = start_addr;

    for (i = 0; i < data_length; i++)
    {
        if (write_data[i] != *(uint8_t __far *)flash_addr)
        {
            ret_value = ENUM_RET_ERR_CHECK_WRITE_DATA;
        }
        flash_addr++;
    }

    return ret_value;
}
#define END_SECTION_SMP_CF
#include "memmap.h"

/**********************************************************************************************************************
 * Function name : r_CheckCFDFSequencerEnd
 *********************************************************************************************************************/
#define START_SECTION_SMP_CMN
#include "memmap.h"
R_RFD_FAR_FUNC e_ret_t r_CheckCFDFSequencerEnd(void)
{
    e_ret_t ret_value = ENUM_RET_STS_OK;
    uint8_t status_flag = 0;

    while (R_RFD_ENUM_RET_STS_BUSY == R_RFD_CheckCFDFSeqEndStep1())
    {
        /* no operation */
    }

    while (R_RFD_ENUM_RET_STS_BUSY == R_RFD_CheckCFDFSeqEndStep2())
    {
        /* no operation */
    }

    R_RFD_GetSeqErrorStatus(&status_flag);

    if (VALUE_U08_MASK0_8BIT != (status_flag & VALUE_U08_MASK1_FSQ_STATUS_ERR_CFDF_SEQUENCER))
    {
        ret_value = ENUM_RET_ERR_CFDF_SEQUENCER;
    }
    else if (VALUE_U08_MASK0_8BIT != (status_flag & VALUE_U08_MASK1_FSQ_STATUS_ERR_ERASE))
    {
        ret_value = ENUM_RET_ERR_ERASE;
    }
    else if (VALUE_U08_MASK0_8BIT != (status_flag & VALUE_U08_MASK1_FSQ_STATUS_ERR_WRITE))
    {
        ret_value = ENUM_RET_ERR_WRITE;
    }
    else if (VALUE_U08_MASK0_8BIT != (status_flag & VALUE_U08_MASK1_FSQ_STATUS_ERR_BLANKCHECK))
    {
        ret_value = ENUM_RET_ERR_BLANKCHECK;
    }
    else
    {
        ret_value = ENUM_RET_STS_OK;
    }

    R_RFD_ClearSeqRegister();

    return (ret_value);
}
#define END_SECTION_SMP_CMN
#include "memmap.h"

/**********************************************************************************************************************
 * Function name : r_CheckExtraSequencerEnd
 *********************************************************************************************************************/
#define START_SECTION_SMP_CMN
#include "memmap.h"
R_RFD_FAR_FUNC e_ret_t r_CheckExtraSequencerEnd(void)
{
    e_ret_t ret_value = ENUM_RET_STS_OK;
    uint8_t status_flag = 0;

    while (R_RFD_ENUM_RET_STS_BUSY == R_RFD_CheckExtraSeqEndStep1())
    {
        /* no operation */
    }

    while (R_RFD_ENUM_RET_STS_BUSY == R_RFD_CheckExtraSeqEndStep2())
    {
        /* no operation */
    }

    R_RFD_GetSeqErrorStatus(&status_flag);

    if (VALUE_U08_MASK0_8BIT != (status_flag & VALUE_U08_MASK1_FSQ_STATUS_ERR_EXTRA_SEQUENCER))
    {
        ret_value = ENUM_RET_ERR_EXTRA_SEQUENCER;
    }
    else if (VALUE_U08_MASK0_8BIT != (status_flag & VALUE_U08_MASK1_FSQ_STATUS_ERR_ERASE))
    {
        ret_value = ENUM_RET_ERR_ERASE;
    }
    else if (VALUE_U08_MASK0_8BIT != (status_flag & VALUE_U08_MASK1_FSQ_STATUS_ERR_WRITE))
    {
        ret_value = ENUM_RET_ERR_WRITE;
    }
    else if (VALUE_U08_MASK0_8BIT != (status_flag & VALUE_U08_MASK1_FSQ_STATUS_ERR_BLANKCHECK))
    {
        ret_value = ENUM_RET_ERR_BLANKCHECK;
    }
    else
    {
        ret_value = ENUM_RET_STS_OK;
    }

    R_RFD_ClearSeqRegister();

    return (ret_value);
}
#define END_SECTION_SMP_CMN
#include "memmap.h"

/**********************************************************************************************************************
 * Function name : r_RequestBootSwap
 *********************************************************************************************************************/
#define START_SECTION_SMP_CF
#include "memmap.h"
R_RFD_FAR_FUNC e_ret_t r_RequestBootSwap(void)
{

    uint16_t f_security_boot = 0;
    e_rfd_boot_cluster_t next_boot_cluster;
    e_rfd_ret_t rfd_ret_status;
    e_ret_t ret_value = ENUM_RET_STS_OK;

    R_RFD_GetSecurityAndBootFlags(&f_security_boot); 
    if (f_security_boot & 0x0100)
    {
        next_boot_cluster = R_RFD_ENUM_BOOT_CLUSTER_1;
    }
    else
    {
        next_boot_cluster = R_RFD_ENUM_BOOT_CLUSTER_0;
    }

    /******************************************************************************************************************
     * Enable data flash access
     *****************************************************************************************************************/

    R_RFD_SetDataFlashAccessMode(R_RFD_ENUM_DF_ACCESS_ENABLE);

    /******************************************************************************************************************
     * Set the code flash programming mode
     *****************************************************************************************************************/

    rfd_ret_status = R_RFD_SetFlashMemoryMode(R_RFD_ENUM_FLASH_MODE_CODE_PROGRAMMING);
    if (R_RFD_ENUM_RET_STS_OK != rfd_ret_status)
    {
        ret_value = ENUM_RET_ERR_MODE_MISMATCHED;
    }

    /******************************************************************************************************************
    *  Set Boot Area
    *****************************************************************************************************************/

    if (ENUM_RET_STS_OK == ret_value)
    {
        R_RFD_SetExtraBootAreaReq(next_boot_cluster); // Execute boot swap after reset
        ret_value = r_CheckExtraSequencerEnd();
    }

    /******************************************************************************************************************
     * Set unprogramable mode
     *****************************************************************************************************************/

    rfd_ret_status = R_RFD_SetFlashMemoryMode(R_RFD_ENUM_FLASH_MODE_UNPROGRAMMABLE);
    if (R_RFD_ENUM_RET_STS_OK != rfd_ret_status)
    {
        ret_value = ENUM_RET_ERR_MODE_MISMATCHED;
    }

    /******************************************************************************************************************
     * Disable data flash access
     *****************************************************************************************************************/

    R_RFD_SetDataFlashAccessMode(R_RFD_ENUM_DF_ACCESS_DISABLE);

    /******************************************************************************************************************
     * CPU Reset
     *****************************************************************************************************************/

    R_RFD_ForceReset(); /* CPU internal reset */

    return ret_value;
}
#define END_SECTION_SMP_CF
#include "memmap.h"

/**********************************************************************************************************************
 * Function name : r_Send_nByte
 *********************************************************************************************************************/
#define START_SECTION_SMP_CF
#include "memmap.h"
MD_STATUS r_Send_nByte(uint8_t *tx_buff, const uint16_t tx_num)
{
    MD_STATUS ret = MD_OK;

    g_UART0_sendend = false;
    ret = R_Config_UART0_Send(tx_buff, tx_num);
    while ((g_UART0_sendend == false) && (ret == MD_OK))
        ;

    return ret;
}
#define END_SECTION_SMP_CF
#include "memmap.h"

/**********************************************************************************************************************
 * Function name : r_SendACK
 *********************************************************************************************************************/
#define START_SECTION_SMP_CF
#include "memmap.h"
MD_STATUS r_SendACK(void)
{
    MD_STATUS ret = MD_OK;
    uint8_t send_buff = 0x01;

    ret = r_Send_nByte(&send_buff, 1);

    return ret;
}
#define END_SECTION_SMP_CF
#include "memmap.h"

/**********************************************************************************************************************
 * Function name : r_CF_MemoryWrite
 *********************************************************************************************************************/
#define START_SECTION_SMP_CF
#include "memmap.h"
R_RFD_FAR_FUNC e_ret_t r_CF_MemoryWrite(uint32_t *write_start_addr, uint32_t write_end_addr, uint8_t __near *write_data)
{
    e_ret_t ret_value = ENUM_RET_STS_OK;

    if (*write_start_addr < write_end_addr)
    {
        DI();
        ret_value = r_CF_WriteVerifySequence(*write_start_addr, WRITE_DATA_SIZE, write_data);
        EI();
        if (ENUM_RET_STS_OK == ret_value)
        {
            *write_start_addr += WRITE_DATA_SIZE;
        }
    }

    return ret_value;
}
#define END_SECTION_SMP_CF
#include "memmap.h"

/**********************************************************************************************************************
 * Function name : r_CF_TempCopy
 *********************************************************************************************************************/
#define START_SECTION_SMP_CF
#include "memmap.h"
R_RFD_FAR_FUNC e_ret_t r_CF_TempCopy(void)
{
    e_ret_t ret_value = ENUM_RET_STS_OK;

    uint32_t tmp_offset = TEMPORARY_START_ADDRESS;
    uint32_t dest_addr = 0;
    int i;
    static uint8_t copy_data[256] = {0};

    for (i = 0; i < EXECUTE_UPDATE_BLOCK_NUMBER; i++)
    {
        MemCpy(copy_data, (void __far *)tmp_offset, WRITE_DATA_SIZE);

        dest_addr = EXECUTE_START_ADDRESS + (WRITE_DATA_SIZE * i);
        ret_value = r_CF_WriteData(dest_addr, WRITE_DATA_SIZE, (uint8_t __near *)(uint16_t)copy_data);

        if (ENUM_RET_STS_OK != ret_value)
        {
            return ret_value;
        }
        tmp_offset += WRITE_DATA_SIZE;
    }

    return ret_value;
}
#define END_SECTION_SMP_CF
#include "memmap.h"

/**********************************************************************************************************************
 End of function main
 *********************************************************************************************************************/